require('datejs');
const each = require('jest-each').default;

const DateUtils = require('../util/DateUtils');

describe('DateUtils', () => {
  describe('Timezones', () => {
    it('should always be GTM-3', () => {
      expect(new Date().getTimezoneOffset()).toBe(180);
    });
  });

  describe('getNowTimeString', () => {
    each([
      // Date now
      [
        {
          company: {
            timeConfiguration: {
              timezone: '-0300',
              hourFormat: 24,
            },
          },
          format: 'd/m/Y',
        },
        `${new Date().format('%R')} (${new Date().format('d/m/Y')}) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '-0600',
              hourFormat: 24,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T12:00:00-03:00'),
        },
        `9:00 (01/01/2020) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '+0000',
              hourFormat: 24,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T12:00:00-03:00'),
        },
        `15:00 (01/01/2020) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '+0100',
              hourFormat: 24,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T11:58:00-03:00'),
        },
        `15:58 (01/01/2020) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '-0600',
              hourFormat: 12,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T12:00:00-03:00'),
        },
        `09:00 AM (01/01/2020) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '+0300',
              hourFormat: 24,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T09:00:00-03:00'),
        },
        `15:00 (01/01/2020) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '+0300',
              hourFormat: 12,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T09:00:00-03:00'),
        },
        `03:00 PM (01/01/2020) `,
      ],
      [
        {
          company: {
            timeConfiguration: {
              timezone: '+1200',
              hourFormat: 24,
            },
          },
          format: 'd/m/Y',
          date: Date.parse('2020-01-01T09:00:00-03:00'),
        },
        `0:00 (02/01/2020) `,
      ],
    ]).it("when the input is '%s'", ({ company, format, date }, expected) => {
      const result = DateUtils.getNowTimeString(company, format, date);
      expect(result).toEqual(expected);
    });
  });
});
