const { ErrorHandlerService } = require('./ErrorHandlerService');

class DialogFlowAccessor {
  constructor(awsLambdaCaller) {
    this.awsLambdaCaller = awsLambdaCaller;
  }

  resolveText(request) {
    return new Promise(resolve => {
      const executor = async () => {
        let intent = null;
        try {
          // Dialogflow
          intent = await this.awsLambdaCaller.call(request, 'asksuite-NLP-dev-dialogFlowNLP');
        } catch (e) {
          const errorData = ErrorHandlerService.accessorErrorHandler(e);
          intent = {
            processingStep: {
              fallbackReason: 'intent_not_found',
              intentFound: null,
              text: request.text,
              language: request.language,
              resolver: 'DIALOG_FLOW',
              errorData,
            },
          };
          console.log('Erro no dialogflow ', e);
        }
        resolve(intent);
      };

      executor();
    });
  }
}

module.exports = DialogFlowAccessor;
